<?php

namespace B2buy\Password\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\Password\Http\Requests\SettingRequest;
use B2buy\Password\Interfaces\SettingRepositoryInterface;
use B2buy\Password\Models\Setting;

/**
 * Resource controller class for setting.
 */
class SettingResourceController extends BaseController
{

    /**
     * Initialize setting resource controller.
     *
     * @param type SettingRepositoryInterface $setting
     *
     * @return null
     */
    public function __construct(SettingRepositoryInterface $setting)
    {
        parent::__construct();
        $this->repository = $setting;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\Password\Repositories\Criteria\SettingResourceCriteria::class);
    }

    /**
     * Display a list of setting.
     *
     * @return Response
     */
    public function index(SettingRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\Password\Repositories\Presenter\SettingPresenter::class)
                ->$function();
        }

        $settings = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('password::setting.names'))
            ->view('password::setting.index', true)
            ->data(compact('settings', 'view'))
            ->output();
    }

    /**
     * Display setting.
     *
     * @param Request $request
     * @param Model   $setting
     *
     * @return Response
     */
    public function show(SettingRequest $request, Setting $setting)
    {

        if ($setting->exists) {
            $view = 'password::setting.show';
        } else {
            $view = 'password::setting.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('password::setting.name'))
            ->data(compact('setting'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new setting.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SettingRequest $request)
    {

        $setting = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('password::setting.name')) 
            ->view('password::setting.create', true) 
            ->data(compact('setting'))
            ->output();
    }

    /**
     * Create new setting.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SettingRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $setting                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('password::setting.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('password/setting/' . $setting->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/password/setting'))
                ->redirect();
        }

    }

    /**
     * Show setting for editing.
     *
     * @param Request $request
     * @param Model   $setting
     *
     * @return Response
     */
    public function edit(SettingRequest $request, Setting $setting)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('password::setting.name'))
            ->view('password::setting.edit', true)
            ->data(compact('setting'))
            ->output();
    }

    /**
     * Update the setting.
     *
     * @param Request $request
     * @param Model   $setting
     *
     * @return Response
     */
    public function update(SettingRequest $request, Setting $setting)
    {
        try {
            $attributes = $request->all();

            $setting->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('password::setting.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('password/setting/' . $setting->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('password/setting/' . $setting->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the setting.
     *
     * @param Model   $setting
     *
     * @return Response
     */
    public function destroy(SettingRequest $request, Setting $setting)
    {
        try {

            $setting->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('password::setting.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('password/setting/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('password/setting/' . $setting->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple setting.
     *
     * @param Model   $setting
     *
     * @return Response
     */
    public function delete(SettingRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('password::setting.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('password/setting'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/password/setting'))
                ->redirect();
        }

    }

    /**
     * Restore deleted settings.
     *
     * @param Model   $setting
     *
     * @return Response
     */
    public function restore(SettingRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('password::setting.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/password/setting'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/password/setting/'))
                ->redirect();
        }

    }

}
